@@ -1,7 +1,7 @@
 /*
  *  db.c
  *
- *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005 by Christian Hamar <krics@linuxforum.hu>
@@ -321,20 +321,55 @@ alpm_list_t SYMEXPORT *alpm_db_search(pmdb_t *db, const alpm_list_t* needles)
 	return(_alpm_db_search(db, needles));
 }
 
+/* Set install reason for a package in db
+ * @param db pointer to the package database
+ * @param name the name of the package
+ * @param reason the new install reason
+ * @return 0 on success, -1 on error (pm_errno is set accordingly)
+ */
+int SYMEXPORT alpm_db_set_pkgreason(pmdb_t *db, const char *name, pmpkgreason_t reason)
+{
+	ALPM_LOG_FUNC;
+
+	/* Sanity checks */
+	ASSERT(handle != NULL, RET_ERR(PM_ERR_HANDLE_NULL, -1));
+	ASSERT(db != NULL && name != NULL, RET_ERR(PM_ERR_WRONG_ARGS, -1));
+
+	pmpkg_t *pkg = _alpm_db_get_pkgfromcache(db, name);
+	if(pkg == NULL) {
+		RET_ERR(PM_ERR_PKG_NOT_FOUND, -1);
+	}
+
+	_alpm_log(PM_LOG_DEBUG, "setting install reason %u for %s/%s\n", reason, db->treename, name);
+	/* read DESC */
+	if(_alpm_db_read(db, pkg, INFRQ_DESC)) {
+		return(-1);
+	}
+	if(pkg->reason == reason) {
+		/* we are done */
+		return(0);
+	}
+	/* set reason (in pkgcache) */
+	pkg->reason = reason;
+	/* write DESC */
+	if(_alpm_db_write(db, pkg, INFRQ_DESC)) {
+		return(-1);
+	}
+
+	return(0);
+}
+
 /** @} */
 
-pmdb_t *_alpm_db_new(const char *dbpath, const char *treename)
+static pmdb_t *_alpm_db_new(const char *treename, int is_local)
 {
 	pmdb_t *db;
-	const size_t pathsize = strlen(dbpath) + strlen(treename) + 2;
 
 	ALPM_LOG_FUNC;
 
 	CALLOC(db, 1, sizeof(pmdb_t), RET_ERR(PM_ERR_MEMORY, NULL));
-	CALLOC(db->path, 1, pathsize, RET_ERR(PM_ERR_MEMORY, NULL));
-
-	sprintf(db->path, "%s%s/", dbpath, treename);
 	STRDUP(db->treename, treename, RET_ERR(PM_ERR_MEMORY, NULL));
+	db->is_local = is_local;
 
 	return(db);
 }
@@ -347,13 +382,44 @@ void _alpm_db_free(pmdb_t *db)
 	_alpm_db_free_pkgcache(db);
 	/* cleanup server list */
 	FREELIST(db->servers);
-	FREE(db->path);
+	FREE(db->_path);
 	FREE(db->treename);
 	FREE(db);
 
 	return;
 }
 
+const char *_alpm_db_path(pmdb_t *db)
+{
+	if(!db) {
+		return(NULL);
+	}
+	if(!db->_path) {
+		const char *dbpath;
+		size_t pathsize;
+
+		dbpath = alpm_option_get_dbpath();
+		if(!dbpath) {
+			_alpm_log(PM_LOG_ERROR, _("database path is undefined\n"));
+			RET_ERR(PM_ERR_DB_OPEN, NULL);
+		}
+
+		if(db->is_local) {
+			pathsize = strlen(dbpath) + strlen(db->treename) + 2;
+			CALLOC(db->_path, 1, pathsize, RET_ERR(PM_ERR_MEMORY, NULL));
+			sprintf(db->_path, "%s%s/", dbpath, db->treename);
+		} else {
+			pathsize = strlen(dbpath) + 5 + strlen(db->treename) + 2;
+			CALLOC(db->_path, 1, pathsize, RET_ERR(PM_ERR_MEMORY, NULL));
+			/* all sync DBs now reside in the sync/ subdir of the dbpath */
+			sprintf(db->_path, "%ssync/%s/", dbpath, db->treename);
+		}
+		_alpm_log(PM_LOG_DEBUG, "database path for tree %s set to %s\n",
+				db->treename, db->_path);
+	}
+	return(db->_path);
+}
+
 int _alpm_db_cmp(const void *d1, const void *d2)
 {
 	pmdb_t *db1 = (pmdb_t *)d1;
@@ -440,7 +506,6 @@ alpm_list_t *_alpm_db_search(pmdb_t *db, const alpm_list_t *needles)
 pmdb_t *_alpm_db_register_local(void)
 {
 	pmdb_t *db;
-	const char *dbpath;
 
 	ALPM_LOG_FUNC;
 
@@ -451,13 +516,7 @@ pmdb_t *_alpm_db_register_local(void)
 
 	_alpm_log(PM_LOG_DEBUG, "registering local database\n");
 
-	dbpath = alpm_option_get_dbpath();
-	if(!dbpath) {
-		_alpm_log(PM_LOG_ERROR, _("database path is undefined\n"));
-			RET_ERR(PM_ERR_DB_OPEN, NULL);
-	}
-
-	db = _alpm_db_new(dbpath, "local");
+	db = _alpm_db_new("local", 1);
 	if(db == NULL) {
 		RET_ERR(PM_ERR_DB_CREATE, NULL);
 	}
@@ -469,8 +528,6 @@ pmdb_t *_alpm_db_register_local(void)
 pmdb_t *_alpm_db_register_sync(const char *treename)
 {
 	pmdb_t *db;
-	const char *dbpath;
-	char path[PATH_MAX];
 	alpm_list_t *i;
 
 	ALPM_LOG_FUNC;
@@ -485,15 +542,7 @@ pmdb_t *_alpm_db_register_sync(const char *treename)
 
 	_alpm_log(PM_LOG_DEBUG, "registering sync database '%s'\n", treename);
 
-	dbpath = alpm_option_get_dbpath();
-	if(!dbpath) {
-		_alpm_log(PM_LOG_ERROR, _("database path is undefined\n"));
-			RET_ERR(PM_ERR_DB_OPEN, NULL);
-	}
-	/* all sync DBs now reside in the sync/ subdir of the dbpath */
-	snprintf(path, PATH_MAX, "%ssync/", dbpath);
-
-	db = _alpm_db_new(path, treename);
+	db = _alpm_db_new(treename, 0);
 	if(db == NULL) {
 		RET_ERR(PM_ERR_DB_CREATE, NULL);
 	}
