@@ -1,7 +1,7 @@
 /*
  *  db.h
  *
- *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2006 by Miklos Vajna <vmiklos@frugalware.org>
@@ -28,30 +28,33 @@
 
 /* Database entries */
 typedef enum _pmdbinfrq_t {
-	INFRQ_BASE = 0x01,
-	INFRQ_DESC = 0x02,
-	INFRQ_DEPENDS = 0x04,
-	INFRQ_FILES = 0x08,
-	INFRQ_SCRIPTLET = 0x10,
-	INFRQ_DELTAS = 0x20,
-	/* ALL should be sum of all above */
+	INFRQ_BASE = 1,
+	INFRQ_DESC = (1 << 1),
+	INFRQ_DEPENDS = (1 << 2),
+	INFRQ_FILES = (1 << 3),
+	INFRQ_SCRIPTLET = (1 << 4),
+	INFRQ_DELTAS = (1 << 5),
+	INFRQ_DSIZE = (1 << 6),
+	/* ALL should be info stored in the package or database */
 	INFRQ_ALL = 0x3F
 } pmdbinfrq_t;
 
 /* Database */
 struct __pmdb_t {
-	char *path;
 	char *treename;
-	unsigned short pkgcache_loaded;
+	/* do not access directly, use _alpm_db_path(db) for lazy access */
+	char *_path;
+	int pkgcache_loaded;
+	int grpcache_loaded;
+	int is_local;
 	alpm_list_t *pkgcache;
-	unsigned short grpcache_loaded;
 	alpm_list_t *grpcache;
 	alpm_list_t *servers;
 };
 
 /* db.c, database general calls */
-pmdb_t *_alpm_db_new(const char *dbpath, const char *treename);
 void _alpm_db_free(pmdb_t *db);
+const char *_alpm_db_path(pmdb_t *db);
 int _alpm_db_cmp(const void *d1, const void *d2);
 alpm_list_t *_alpm_db_search(pmdb_t *db, const alpm_list_t *needles);
 pmdb_t *_alpm_db_register_local(void);
