@@ -1,7 +1,7 @@
 /*
  *  delta.c
  *
- *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2007-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -71,34 +71,16 @@ off_t SYMEXPORT alpm_delta_get_size(pmdelta_t *delta)
 
 /** @} */
 
-static alpm_list_t *delta_graph_init(alpm_list_t *deltas)
+static alpm_list_t *graph_init(alpm_list_t *deltas, int reverse)
 {
 	alpm_list_t *i, *j;
 	alpm_list_t *vertices = NULL;
 	/* create the vertices */
 	for(i = deltas; i; i = i->next) {
-		char *fpath, *md5sum;
 		pmgraph_t *v = _alpm_graph_new();
 		pmdelta_t *vdelta = i->data;
 		vdelta->download_size = vdelta->delta_size;
 		v->weight = LONG_MAX;
-
-		/* determine whether the delta file already exists */
-		fpath = _alpm_filecache_find(vdelta->delta);
-		md5sum = alpm_compute_md5sum(fpath);
-		if(fpath && md5sum && strcmp(md5sum, vdelta->delta_md5) == 0) {
-			vdelta->download_size = 0;
-		}
-		FREE(fpath);
-		FREE(md5sum);
-
-		/* determine whether a base 'from' file exists */
-		fpath = _alpm_filecache_find(vdelta->from);
-		if(fpath) {
-			v->weight = vdelta->download_size;
-		}
-		FREE(fpath);
-
 		v->data = vdelta;
 		vertices = alpm_list_add(vertices, v);
 	}
@@ -119,7 +101,8 @@ static alpm_list_t *delta_graph_init(alpm_list_t *deltas)
 			 *     3_to_4
 			 * If J 'from' is equal to I 'to', then J is a child of I.
 			 * */
-			if(strcmp(d_j->from, d_i->to) == 0) {
+			if((!reverse && strcmp(d_j->from, d_i->to) == 0) ||
+					(reverse && strcmp(d_j->to, d_i->from) == 0)) {
 				v_i->children = alpm_list_add(v_i->children, v_j);
 			}
 		}
@@ -128,8 +111,36 @@ static alpm_list_t *delta_graph_init(alpm_list_t *deltas)
 	return(vertices);
 }
 
-static off_t delta_vert(alpm_list_t *vertices,
-		const char *to, alpm_list_t **path) {
+static void graph_init_size(alpm_list_t *vertices)
+{
+	alpm_list_t *i;
+
+	for(i = vertices; i; i = i->next) {
+		char *fpath, *md5sum;
+		pmgraph_t *v = i->data;
+		pmdelta_t *vdelta = v->data;
+
+		/* determine whether the delta file already exists */
+		fpath = _alpm_filecache_find(vdelta->delta);
+		md5sum = alpm_compute_md5sum(fpath);
+		if(fpath && md5sum && strcmp(md5sum, vdelta->delta_md5) == 0) {
+			vdelta->download_size = 0;
+		}
+		FREE(fpath);
+		FREE(md5sum);
+
+		/* determine whether a base 'from' file exists */
+		fpath = _alpm_filecache_find(vdelta->from);
+		if(fpath) {
+			v->weight = vdelta->download_size;
+		}
+		FREE(fpath);
+	}
+}
+
+
+static void dijkstra(alpm_list_t *vertices)
+{
 	alpm_list_t *i;
 	pmgraph_t *v;
 	while(1) {
@@ -165,9 +176,14 @@ static off_t delta_vert(alpm_list_t *vertices,
 
 		}
 	}
+}
 
-	v = NULL;
+static off_t shortest_path(alpm_list_t *vertices, const char *to, alpm_list_t **path)
+{
+	alpm_list_t *i;
+	pmgraph_t *v = NULL;
 	off_t bestsize = 0;
+	alpm_list_t *rpath = NULL;
 
 	for(i = vertices; i; i = i->next) {
 		pmgraph_t *v_i = i->data;
@@ -181,7 +197,6 @@ static off_t delta_vert(alpm_list_t *vertices,
 		}
 	}
 
-	alpm_list_t *rpath = NULL;
 	while(v != NULL) {
 		pmdelta_t *vdelta = v->data;
 		rpath = alpm_list_add(rpath, vdelta);
@@ -219,9 +234,10 @@ off_t _alpm_shortest_delta_path(alpm_list_t *deltas,
 
 	_alpm_log(PM_LOG_DEBUG, "started delta shortest-path search for '%s'\n", to);
 
-	vertices = delta_graph_init(deltas);
-
-	bestsize = delta_vert(vertices, to, &bestpath);
+	vertices = graph_init(deltas, 0);
+	graph_init_size(vertices);
+	dijkstra(vertices);
+	bestsize = shortest_path(vertices, to, &bestpath);
 
 	_alpm_log(PM_LOG_DEBUG, "delta shortest-path search complete : '%jd'\n", (intmax_t)bestsize);
 
@@ -232,6 +248,45 @@ off_t _alpm_shortest_delta_path(alpm_list_t *deltas,
 	return(bestsize);
 }
 
+static alpm_list_t *find_unused(alpm_list_t *deltas, const char *to, off_t quota)
+{
+	alpm_list_t *unused = NULL;
+	alpm_list_t *vertices;
+	alpm_list_t *i;
+	vertices = graph_init(deltas, 1);
+
+	for(i = vertices; i; i = i->next) {
+		pmgraph_t *v = i->data;
+		pmdelta_t *vdelta = v->data;
+		if(strcmp(vdelta->to, to) == 0)
+		{
+			v->weight = vdelta->download_size;
+		}
+	}
+	dijkstra(vertices);
+	for(i = vertices; i; i = i->next) {
+		pmgraph_t *v = i->data;
+		pmdelta_t *vdelta = v->data;
+		if(v->weight > quota) {
+			unused = alpm_list_add(unused, vdelta->delta);
+		}
+	}
+	alpm_list_free_inner(vertices, _alpm_graph_free);
+	alpm_list_free(vertices);
+	return(unused);
+}
+
+alpm_list_t SYMEXPORT *alpm_pkg_unused_deltas(pmpkg_t *pkg)
+{
+	off_t pkgsize = alpm_pkg_get_size(pkg);
+	alpm_list_t *unused = find_unused(
+			alpm_pkg_get_deltas(pkg),
+			alpm_pkg_get_filename(pkg),
+			pkgsize * MAX_DELTA_RATIO);
+	return(unused);
+}
+
+
 /** Parses the string representation of a pmdelta_t object.
  * This function assumes that the string is in the correct format.
  * This format is as follows:
@@ -282,7 +337,7 @@ pmdelta_t *_alpm_delta_parse(char *line)
 	tmp2 = tmp;
 	STRDUP(delta->to, tmp2, RET_ERR(PM_ERR_MEMORY, NULL));
 
-	_alpm_log(PM_LOG_DEBUG, "delta : %s %s '%lld'\n", delta->from, delta->to, (long long)delta->delta_size);
+	_alpm_log(PM_LOG_DEBUG, "delta : %s %s '%jd'\n", delta->from, delta->to, (intmax_t)delta->delta_size);
 
 	return(delta);
 }
