@@ -1,7 +1,7 @@
 /*
  *  error.c
  *
- *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -117,8 +117,10 @@ const char SYMEXPORT *alpm_strerror(int err)
 			return _("cannot remove all files for package");
 		case PM_ERR_PKG_INVALID_NAME:
 			return _("package filename is not valid");
+		case PM_ERR_PKG_INVALID_ARCH:
+			return _("package architecture is not valid");
 		case PM_ERR_PKG_REPO_NOT_FOUND:
-			return _("no such repository");
+			return _("could not find repository for target");
 		/* Deltas */
 		case PM_ERR_DLT_INVALID:
 			return _("invalid or corrupted delta");
