@@ -176,7 +176,7 @@ static void version(void)
 {
 	printf("\n");
 	printf(" .--.                  Pacman v%s - libalpm v%s\n", PACKAGE_VERSION, alpm_version());
-	printf("/ _.-' .-.  .-.  .-.   Copyright (C) 2006-2010 Pacman Development Team\n");
+	printf("/ _.-' .-.  .-.  .-.   Copyright (C) 2006-2011 Pacman Development Team\n");
 	printf("\\  '-. '-'  '-'  '-'   Copyright (C) 2002-2006 Judd Vinet\n");
 	printf(" '--'\n");
 	printf(_("                       This program may be freely redistributed under\n"
@@ -218,10 +218,10 @@ static void setarch(const char *arch)
 	if (strcmp(arch, "auto") == 0) {
 		struct utsname un;
 		uname(&un);
-		pm_printf(PM_LOG_DEBUG, "config: architecture: %s\n", un.machine);
+		pm_printf(PM_LOG_DEBUG, "config: Architecture: %s\n", un.machine);
 		alpm_option_set_arch(un.machine);
 	} else {
-		pm_printf(PM_LOG_DEBUG, "config: architecture: %s\n", arch);
+		pm_printf(PM_LOG_DEBUG, "config: Architecture: %s\n", arch);
 		alpm_option_set_arch(arch);
 	}
 }
